//
//  SentryCrashReportFixer.c
//
//  Created by Karl Stenerud on 2016-11-07.
//
//  Copyright (c) 2012 Karl Stenerud. All rights reserved.
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall remain in place
// in this source code.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
//

#ifndef HDR_SentryCrashReportFixer_h
#define HDR_SentryCrashReportFixer_h

#ifdef __cplusplus
extern "C" {
#endif

/** Fixes up fields in a crash report that could not be fixed up at crash time.
 * Some fields, such a mangled fields and dates, cannot be fixed up at crash
 * time because the function calls needed to do it are not async-safe.
 *
 * @param crashReport A raw report loaded from disk.
 *
 * @return A fixed up crash report.
 *         MEMORY MANAGEMENT WARNING: User is responsible for calling free() on
 * the returned value.
 */
char *sentrycrashcrf_fixupCrashReport(const char *crashReport);

#ifdef __cplusplus
}
#endif

#endif // HDR_SentryCrashReportFixer_h
